import { createContext, useContext, useEffect } from 'react';
import { toJS } from 'mobx';
import { observer } from 'mobx-react';
import { DATE_FORMAT, SMALL_DATETIME_FORMAT, stores_Context } from './../../config';
import { SearchOutlined } from '@ant-design/icons';
import { Form, Row, Col, Select, Button, Space, DatePicker, Input, InputNumber } from 'antd';
import moment from 'moment';
// import locale from 'antd/es/date-picker/locale/zh_CN';
import BusinessSelect from './BusinessSelect';
import BusinessUnitSelect from './BusinessUnitSelect';
import GroupSelect from './GroupSelect';
import SiteSelect from './SiteSelect';
import DateTypeSelect from './DataTypeSelect';
import DatePickerCharts from './DatePickerCharts';
import YearPickerCharts from './YearPickerCharts';
import SearchInput from './Input';
import { objectMapper, at, empty, isEmpty } from './../../utils/commons';
import { departureDateTypes } from './../../libs/ht';

import './search.css';
import HotelStarSelect from './HotelStarSelect';

const EditableContext = createContext();
const Option = Select.Option;

/**
 * 搜索表单
 * @property defaultValue { initialValue, fieldProps, hides, shows, sort }
 * * {object} initialValue 默认值
 * * {object} fieldProps 表单项属性
 * * {array} hides 隐藏的表单项
 * * {array} shows 显示的表单项
 * * {object} sort 表单项排序
 * @property onSubmit
 */
export default observer((props) => {
  const { date_picker_store: searchFormStore } = useContext(stores_Context);
  const [form] = Form.useForm();
  const { sort, initialValue, hides, shows, fieldProps } = {
    sort: '',
    initialValue: '',
    fieldProps: '',
    hides: [],
    shows: [],
    ...props.defaultValue,
  };
  const { onSubmit, confirmText } = props;


  const formValuesMapper = (values) => {
    const destinationObject = {
      'DateType': {
        key: 'DateType',
        transform: (value) => value?.key || '',
        default: '',
      },
      'departureDateType': {
        key: 'DateType',
        transform: (value) => value?.key || '',
        default: '',
      },
      'HTBusinessUnits': {
        key: 'HTBusinessUnits',
        transform: (value) => {
          return Array.isArray(value) ? value.map((ele) => ele.key).join(',') : value ? (!isNaN(parseInt(value.key), 10) ? value.key : '') : '';
        },
        default: '',
      },
      'businessUnits': {
        key: 'businessUnits',
        transform: (value) => {
          return isEmpty(value) ? 'ALL': Array.isArray(value) ? value.map((ele) => ele.value).join(',') : value ? value.value : '';
        },
        default: '',
      },
      'DepartmentList': {
        key: 'DepartmentList',
        transform: (value) => {
          return isEmpty(value) ? 'ALL': Array.isArray(value) ? value.map((ele) => ele.key).join(',') : value ? value.key : '';
        },
        default: '',
      },
      'WebCode': {
        key: 'WebCode',
        transform: (value) => {
          return isEmpty(value) ? 'ALL': Array.isArray(value) ? value.map((ele) => ele.key).filter(ele => ele !== 'ALL').join(',') : value ? value.key : '';
        },
        default: '',
      },
      'IncludeTickets': {
        key: 'IncludeTickets',
        transform: (value) => value?.key || '',
        default: '',
      },
      'operator': {
        key: 'operator',
        // transform: (value) => value?.key || '',
        transform: (value) => Array.isArray(value) ? value.map((ele) => ele.key).join(',') : value ? (!isNaN(parseInt(value.key), 10) ? value.key : '') : '',
        default: '',
      },
      'date': {
        key: 'date',
        transform: (value) => (value ? moment(value).format(SMALL_DATETIME_FORMAT) : undefined),
        default: '',
      },
      'applyDate': [
        {
          key: 'Date1',
          transform: (value) => (value === '' || !Array.isArray(value) ? undefined : moment(value[0]).format(DATE_FORMAT)),
          default: '',
        },
        {
          key: 'Date2',
          transform: (value) => (value === '' || !Array.isArray(value) ? undefined : moment(value[1]).format(SMALL_DATETIME_FORMAT)),
          default: '',
        },
      ],
      'applyDate2': [
        {
          key: 'DateDiff1',
          transform: (value) => (value === '' || !Array.isArray(value) ? undefined : value[0] ? moment(value[0]).format(DATE_FORMAT) : undefined),
          default: '',
        },
        {
          key: 'DateDiff2',
          transform: (value) => (value === '' || !Array.isArray(value) ? undefined : value[1] ? moment(value[1]).format(SMALL_DATETIME_FORMAT) : undefined),
          default: '',
        },
      ],
      'year': [
        {
          key: 'Date1',
          transform: (value) => (value ? moment(value).format(`YYYY-01-01`) : undefined),
          default: '',
        },
        {
          key: 'Date2',
          transform: (value) => (value ? moment(value).format(`YYYY-12-31 23:59:00`) : undefined),
          default: '',
        },
      ],
      'yearDiff': [
        {
          key: 'DateDiff1',
          transform: (value) => (value ? moment(value).format(`YYYY-01-01`) : undefined),
          default: '',
        },
        {
          key: 'DateDiff2',
          transform: (value) => (value ? moment(value).format(`YYYY-12-31 23:59:00`) : undefined),
          default: '',
        },
      ],
      'country': {
        key: 'country',
        transform: (value) => value?.key || '',
        default: '',
      },
      'city': {
        key: 'city',
        transform: (value) => value?.key || '',
        default: '',
      },
      'billtype': {
        key: 'billtype',
        transform: (value) => isEmpty(value) ? 'ALL' :  value?.key || '',
        default: '',
      },
      'agency': {
        key: 'agency',
        transform: (value) => value?.key || '',
        default: '',
      },
      'countryArea': {
        key: 'countryArea',
        transform: (value) => value?.value || value?.key || '',
        default: '',
      },
      'orderStatus': {
        key: 'orderStatus',
        transform: (value) => value?.value || value?.key || '',
        default: '',
      },
      'cruiseDirection': {
        key: 'cruiseDirection',
        transform: (value) => value?.value || '',
        default: '',
      },
      'cruiseBookType': {
        key: 'cruiseBookType',
        transform: (value) => value?.value || '',
        default: '',
      },
      'hotelBookType': {
        key: 'hotelBookType',
        transform: (value) => value?.value || '',
        default: '',
      },
      'hotelRecommandRate': {
        key: 'hotelRecommandRate',
        transform: (value) => value?.value || '',
        default: '',
      },
      'hotelStar': {
        key: 'hotelStar',
        transform: (value) => value?.value || '',
        default: '',
      },
    };
    let dest = {};
    const { departureDateType, applyDate, applyDate2, year, yearDiff, dates, months, date, ...omittedValue } = values;
    dest = { ...omittedValue, ...objectMapper(values, destinationObject) };
    for (const key in dest) {
      if (Object.prototype.hasOwnProperty.call(dest, key)) {
        dest[key] = typeof dest[key] === 'string' ? (dest[key] || '').trim() : dest[key];
      }
    }
    // omit empty
    Object.keys(dest).forEach((key) => (dest[key] == null || dest[key] === '' || dest[key].length === 0) && delete dest[key]);
    return dest;
  };

  useEffect(() => {
    const dest = formValuesMapper(searchFormStore.formValues);
    searchFormStore.setFormValuesToSub(dest);
    return () => {};
  }, []);

  const onFinish = (values) => {
    console.log('Received values of form, origin form value: ', values);
    const dest = formValuesMapper(values);
    console.log('form value send to onSubmit:', dest);
    const str = new URLSearchParams(dest).toString();
    searchFormStore.setFormValues(values);
    searchFormStore.setFormValuesToSub(dest);
    if (typeof onSubmit === 'function') {
      onSubmit(null, dest, values, str);
    }
  };

  const onReset = () => {
    form.setFieldsValue({
      // 'DateType': undefined,
    });
  };
  const onValuesChange = (...args) => {
    const [changedValues, allValues] = args;

    const dest = formValuesMapper(allValues);
    searchFormStore.setFormValues(allValues);
    searchFormStore.setFormValuesToSub(dest);
    // console.log('form onValuesChange', Object.keys(changedValues), args);
  };

  return (
    // layout="inline"
    <Form form={form} name="advanced_search" className="orders-search-form" onFinish={onFinish} onValuesChange={onValuesChange}>
      <EditableContext.Provider value={form}>
        <Row gutter={10} style={{ background: '#f9fafa', margin: '0px 0px 10px 0px', padding: '16px 8px 0 8px', boxShadow: '0px 0px 3px 0px rgba(0,0,0,0.15)' }}>
          {getFields({ sort, initialValue, hides, shows, fieldProps, form })}
          {/* 'textAlign': 'right' */}
          <Col flex="1 0 90px" style={{ padding: '0px 5px', display: 'flex', justifyContent: 'flex-end', alignItems: 'flex-start' }}>
            <Space align="center">
              <Button size={'middle'} type="primary" icon={<SearchOutlined />} htmlType="submit">
                {confirmText || '统计'}
              </Button>
              {/* <Button size="small" onClick={onReset}>
                重置
              </Button> */}
            </Space>
          </Col>
        </Row>
      </EditableContext.Provider>
    </Form>
  );
});

function getFields(props) {
  const { fieldProps, form } = props;
  const bigCol = 4 * 2;
  const midCol = 6;
  const layoutProps = {
    gutter: { xs: 8, sm: 8, lg: 16 },
    lg: { span: 4 },
    md: { span: 8 },
    sm: { span: 12 },
    xs: { span: 24 },
  };
  const item = (name, sort = 0, render, col) => {
    const customCol = col || 4;
    const mdCol = customCol * 2;
    return {
      'key': '',
      sort,
      name,
      render,
      'hide': false,
      'col': { lg: { span: customCol }, md: { span: mdCol < 8 ? 10 : mdCol}, flex: mdCol < 8 ? "1 0" : "" },
    };
  };
  let baseChildren = [];
  baseChildren = [
    item(
      'keyword', // 关键词搜索 {...fieldComProps.keyword}
      99,
      <Form.Item name="keyword" {...fieldProps.keyword}>
        <Input allowClear {...fieldProps.keyword} />
      </Form.Item>,
      fieldProps?.keyword?.col || 6
    ),
    item(
      'agency',
      99,
      <Form.Item name={'agency'}>
        <SearchInput autoGet url="/service-web/QueryData/GetVEIName" map={{ 'CAV_VEI_SN': 'key', 'VEI2_CompanyBN': 'label' }} resultkey={'result1'} placeholder="所有地接社" {...fieldProps.agency} />
      </Form.Item>
    ),
    item(
      'billtype',
      99,
      <Form.Item name={'billtype'}>
        <SearchInput
          autoGet
          url="/service-web/QueryData/GetCreditCardBillType"
          map={{ 'cb_billtype': ['key', 'label'] }}
          // map={{ 'cb_billtype': 'key' }}
          resultkey={'billtype'}
          placeholder="所有账单类型"
        />
      </Form.Item>
    ),
    item(
      'HTBusinessUnits',
      99,
      <Form.Item name={`HTBusinessUnits`} initialValue={at(props, 'initialValue.HTBusinessUnits')[0] || undefined}>
        <BusinessUnitSelect {...fieldProps.HTBusinessUnits} labelInValue={true} />
      </Form.Item>
    ),
    item(
      'businessUnits',
      99,
      <Form.Item name={`businessUnits`} initialValue={at(props, 'initialValue.businessUnits')[0] || undefined}>
        <BusinessSelect {...fieldProps.businessUnits} labelInValue={true} />
      </Form.Item>
    ),
    item(
      'DepartmentList',
      99,
      <Form.Item
        name={`DepartmentList`}
        initialValue={at(props, 'initialValue.DepartmentList')[0] || (fieldProps?.DepartmentList?.show_all ? { key: 'ALL', label: '所有小组' } : undefined)}
        rules={[{ required: true, message: '选择小组' }]}
      >
        <GroupSelect {...fieldProps.DepartmentList} labelInValue={true} />
      </Form.Item>,
      fieldProps?.DepartmentList?.col
    ),
    item(
      'WebCode',
      99,
      <Form.Item name={`WebCode`} initialValue={at(props, 'initialValue.WebCode')[0] || (fieldProps?.WebCode?.show_all ? { key: 'ALL', label: '所有来源' } : undefined)}>
        <SiteSelect {...fieldProps.WebCode} labelInValue={true} />
      </Form.Item>,
      fieldProps?.WebCode?.col
    ),
    item(
      'IncludeTickets',
      99,
      //  value={orders_store.include_tickets} onChange={orders_store.handleChange_include_tickets}
      <Form.Item name={`IncludeTickets`} initialValue={at(props, 'initialValue.IncludeTickets')[0] || { key: '1', label: '含门票' }}>
        <Select style={{ width: '100%' }} placeholder="是否含门票" labelInValue>
          <Option key="1" value="1">
            含门票
          </Option>
          <Option key="0" value="0">
            不含门票
          </Option>
        </Select>
      </Form.Item>,
      3
    ),
    item(
      'countryArea',
      99,
      <Form.Item name={`countryArea`} initialValue={at(props, 'initialValue.countryArea')[0] || (fieldProps?.countryArea?.show_all ? { key: 'all', label: '国内外' } : undefined)}>
        <Select style={{ width: '100%' }} placeholder="国内外" labelInValue allowClear={fieldProps?.countryArea?.show_all || false}>
          {fieldProps?.countryArea?.show_all && (
            <Option key="all" value="" disabled>
              国内外
            </Option>
          )}
          <Option key="china" value="china">
            国内
          </Option>
          <Option key="foreign" value="foreign">
            国外
          </Option>
        </Select>
      </Form.Item>,
      3
    ),
    item(
      'orderStatus',
      99,
      <Form.Item name={`orderStatus`} initialValue={at(props, 'initialValue.orderStatus')[0] || (fieldProps?.orderStatus?.show_all ? { key: '-1', label: '成行状态' } : undefined)}>
        <Select style={{ width: '100%' }} placeholder="成行状态" labelInValue allowClear>
          {fieldProps?.orderStatus?.show_all && (
            <Select.Option key="-1" value="-1" disabled>
              成行状态
            </Select.Option>
          )}
          <Select.Option key="已成行" value="1">
            已成行
          </Select.Option>
          <Select.Option key="未成行" value="0">
            未成行
          </Select.Option>
        </Select>
      </Form.Item>,
      3
    ),
    //
    item(
      'DateType',
      99,
      <Form.Item name={`DateType`} initialValue={at(props, 'initialValue.DateType')[0] || { key: 'applyDate', label: '提交日期' }}>
        <DateTypeSelect labelInValue={true} disabledkeys={fieldProps?.DateType?.disabledKeys || []} />
      </Form.Item>,
      fieldProps?.DateType?.col || 3
    ),
    item(
      'departureDateType',
      99,
      <Form.Item name={`departureDateType`} initialValue={at(props, 'initialValue.departureDateType')[0] || { key: 'departureDate', label: '抵达日期' }}>
        <Select labelInValue={true} style={{ width: '100%' }} placeholder="选择日期类型">
          {departureDateTypes.map((ele) => (
            <Select.Option key={ele.key} value={ele.key} disabled={fieldProps?.departureDateType?.disabledKeys.includes(ele.key)}>
              {ele.label}
            </Select.Option>
          ))}
        </Select>
      </Form.Item>,
      fieldProps?.departureDateType?.col || 3
    ),
    item(
      'years',
      99,
      <Form.Item>
        {/* <DatePicker picker="year" placeholder='年份' /> */}
        <YearPickerCharts {...fieldProps.years} />
      </Form.Item>,
      3
    ),
    item(
      'months',
      99,
      <Form.Item>
        <DatePicker picker="month" placeholder="月份" />
      </Form.Item>,
      3
    ),
    item(
      'dates',
      99,
      <Form.Item>
        <DatePickerCharts isform={true} {...fieldProps.dates} form={form} />
      </Form.Item>,
      fieldProps?.dates?.col || midCol
    ),
    item(
      'date',
      99,
      <Form.Item name={`date`} initialValue={at(props, 'initialValue.date')[0]}>
        <DatePicker picker="date" placeholder="日期" />
      </Form.Item>,
      fieldProps?.date?.col || midCol
    ),
    item(
      'operator',
      99,
      <Form.Item name={'operator'} dependencies={['DepartmentList']}>
        <SearchInput
          {...fieldProps.operator}
          autoGet
          url="/service-Analyse2/GetOperatorInfo"
          map={{ 'op_id': 'key', 'cn_name': 'label' }}
          resultkey={'result'}
          placeholder="输入搜索顾问: 中/英名字"
          dependenciesFun={() => {
            let dependenciesValue = '';
            if (Array.isArray(form.getFieldValue('DepartmentList'))) {
              dependenciesValue = form
                .getFieldValue('DepartmentList')
                .map((e) => e.value)
                .join(',')
                .replace('ALL', '');
            } else {
              dependenciesValue = (form.getFieldValue('DepartmentList')?.value || '').replace('ALL', '');
            }
            return { dept_id: dependenciesValue, ...(fieldProps?.operator?.param || {}) };
          }}
        />
      </Form.Item>
    ),
    item(
      'country',
      99,
      <Form.Item name={'country'}>
        <SearchInput autoGet url="/service-Analyse2/GetCountryInfo" map={{ 'c_id': 'key', 'cn_name': 'label' }} resultkey={'result'} placeholder="输入搜索国籍: 中/英名字" />
      </Form.Item>
    ),
    item(
      'city',
      99,
      <Form.Item name={'city'}>
        <SearchInput autoGet url="/service-Analyse2/GetDestinationInfo" map={{ 'c_id': 'key', 'cn_name': 'label' }} resultkey={'result'} placeholder="输入搜索城市: 中/英名字" />
      </Form.Item>
    ),
    item(
      'globalCity',
      99,
      <Form.Item name={'globalCity'}>
        <SearchInput autoGet url="/service-Analyse2/GetGlobalDestinationInfo" map={{ 'c_id': 'key', 'cn_name': 'label' }} resultkey={'result'} placeholder="输入搜索城市: 中/英名字" />
      </Form.Item>
    ),
    item(
      'cruiseDirection',
      99,
      <Form.Item name={`cruiseDirection`} initialValue={at(props, 'initialValue.cruiseDirection')[0] || undefined}>
        <Select style={{ width: '100%' }} placeholder="上下水" labelInValue allowClear>
          {fieldProps?.cruiseDirection?.show_all && (
            <Option key="all" value="" disabled>
              上下水
            </Option>
          )}
          <Option key="1" value="1">
            上水
          </Option>
          <Option key="2" value="2">
            下水
          </Option>
          {/* <Option key="long" value="long">
            长线
          </Option> */}
        </Select>
      </Form.Item>,
      3
    ),
    item(
      'cruiseBookType',
      99,
      <Form.Item name={`cruiseBookType`} initialValue={at(props, 'initialValue.cruiseBookType')[0] || (fieldProps?.cruiseBookType?.show_all ? { key: '-1', label: '预定类型' } : undefined)}>
        <Select style={{ width: '100%' }} placeholder="预定类型" labelInValue allowClear>
          {fieldProps?.cruiseBookType?.show_all && (
            <Option key="-1" value="-1" disabled>
              预定类型
            </Option>
          )}
          <Option key="1" value="1">
            单订三峡
          </Option>
          <Option key="0" value="0">
            含行程
          </Option>
        </Select>
      </Form.Item>,
      3
    ),
    item(
      'roomsRange',
      99,
      <Form.Item noStyle>
        <Input.Group compact>
          <Form.Item name={'RoomNumStart'} noStyle>
            <InputNumber style={{ width: 'calc(50% - 15px)', textAlign: 'center' }} placeholder="房间数" />
          </Form.Item>
          <Input style={{ width: 30, borderLeft: 0, borderRight: 0, pointerEvents: 'none', }} placeholder="~" disabled />
          <Form.Item name={'RoomNumEnd'} noStyle>
            <InputNumber style={{ width: 'calc(50% - 15px)', textAlign: 'center', borderLeft: 0 }} placeholder="房间数" />
          </Form.Item>
        </Input.Group>
      </Form.Item>,
      fieldProps?.roomsRange?.col || 4
    ),
    item(
      'personRange',
      99,
      <Form.Item noStyle>
        <Input.Group compact>
          <Form.Item name={'PersonNumStart'} noStyle>
            <InputNumber style={{ width: 'calc(50% - 15px)', textAlign: 'center' }} placeholder="人数" />
          </Form.Item>
          <Input style={{ width: 30, borderLeft: 0, borderRight: 0, pointerEvents: 'none', }} placeholder="~" disabled />
          <Form.Item name={'PersonNumEnd'} noStyle>
            <InputNumber style={{ width: 'calc(50% - 15px)', textAlign: 'center', borderLeft: 0 }} placeholder="人数" />
          </Form.Item>
        </Input.Group>
      </Form.Item>,
      fieldProps?.personRange?.col || 4
    ),
    item(
      'hotelBookType',
      99,
      <Form.Item name={`hotelBookType`} initialValue={at(props, 'initialValue.hotelBookType')[0] || (fieldProps?.hotelBookType?.show_all ? { key: 'all', label: '预定类型' } : undefined)}>
        <Select style={{ width: '100%' }} placeholder="预定类型" labelInValue allowClear={fieldProps?.hotelBookType?.show_all || false}>
          {fieldProps?.hotelBookType?.show_all && (
            <Option key="all" value="" disabled>
              预定类型
            </Option>
          )}
          <Option key="1" value="1">
            代订
          </Option>
          <Option key="0" value="0">
            自订
          </Option>
        </Select>
      </Form.Item>,
      3
    ),
    item(
      'hotelRecommandRate',
      99,
      <Form.Item name={`hotelRecommandRate`} initialValue={at(props, 'initialValue.hotelRecommandRate')[0] || (fieldProps?.hotelRecommandRate?.show_all ? { key: 'all', label: '推荐等级' } : undefined)}>
        <Select style={{ width: '100%' }} placeholder="推荐等级" labelInValue allowClear={fieldProps?.hotelRecommandRate?.show_all || false}>
          {fieldProps?.hotelRecommandRate?.show_all && (
            <Option key="all" value="" disabled>
              推荐等级
            </Option>
          )}
          <Option key="1" value="1">
            主推
          </Option>
          <Option key="0" value="0">
            非主推
          </Option>
        </Select>
      </Form.Item>,
      3
    ),
    item(
      'hotelStar',
      99,
      <Form.Item name={`hotelStar`} initialValue={at(props, 'initialValue.hotelStar')[0] || undefined}>
        <HotelStarSelect {...fieldProps.hotelStar} labelInValue={true} />
      </Form.Item>
    ),
  ];
  baseChildren = baseChildren
    .map((x) => {
      x.hide = false;
      if (props.sort === undefined) {
        return x;
      }
      const tmpSort = props.sort;
      for (const key in tmpSort) {
        if (Object.prototype.hasOwnProperty.call(tmpSort, key)) {
          if (x.name === key) {
            x.sort = tmpSort[key];
          }
        }
      }
      return x;
    })
    .map((x) => {
      if (props.hides.length === 0 && props.shows.length === 0) {
        return x;
      }
      if (props.hides.length === 0) {
        x.hide = !props.shows.includes(x.name);
      } else if (props.shows.length === 0) {
        x.hide = props.hides.includes(x.name);
      }
      return x;
    })
    .filter((x) => !x.hide)
    .sort((a, b) => {
      return a.sort < b.sort ? -1 : 1;
    });
  const children = [];
  const leftStyle = {}; // { borderRight: '1px solid #dedede' };
  for (let i = 0; i < baseChildren.length; i++) {
    let style = { padding: '0px 2px' };
    style = i % 2 === 0 && baseChildren[i].col === 12 ? { ...style, ...leftStyle } : style;
    style = !baseChildren[i].hide ? { ...style, display: 'block' } : { ...style, display: 'none' };
    const Item = (
      <Col key={String(i)} style={style} {...layoutProps} {...baseChildren[i].col}>
        {baseChildren[i].render}
      </Col>
    );
    children.push(Item);
  }
  return children;
}
